#pragma semicolon 1

#include <sourcemod>
#include <sdktools>
#include <sdkhooks>
#include <zombiereloaded>
#include <shop>
#include <cstrike>
#include <colors>

#define PLUGIN_VERSION	"2.0.1"

#define CATEGORY	"zr"
#define ITEM	"infect_nade"

new Handle:g_hDistance, Float:g_fDistance,
	Handle:g_hExplodeDelay, Float:g_fExplodeDelay,
	Handle:g_hColor, g_iColor[4],
Handle:g_hSound, String:g_sSound[PLATFORM_MAX_PATH] = "ui/freeze_cam.wav";

new Handle:g_hPrice, iPrice;

new ItemId:id;

new g_beamsprite, g_halosprite;

public Plugin:myinfo =
{
	name = "[SHOP:ZR] Infect Nade",
	author = "FrozDark (HLModders LLC)",
	description = "Adds an infection nade to the shop",
	version = PLUGIN_VERSION,
	url = "http://www.hlmod.ru/"
};

public OnPluginStart()
{
	CreateConVar("shop_zr_infect_nade_version", PLUGIN_VERSION, "[Shop] Infection nade version", FCVAR_SPONLY|FCVAR_REPLICATED|FCVAR_NOTIFY|FCVAR_CHEAT|FCVAR_DONTRECORD);
	
	g_hDistance = CreateConVar("shop_zr_infect_nade_distance", "600.0", "Infection distance", _, true, 100.0);
	g_fDistance = GetConVarFloat(g_hDistance);
	HookConVarChange(g_hDistance, OnConVarChange);
	
	g_hExplodeDelay = CreateConVar("shop_zr_infect_nade_explode_delay", "1.3", "Time before explosion", _, true, 0.3);
	g_fExplodeDelay = GetConVarFloat(g_hExplodeDelay);
	HookConVarChange(g_hExplodeDelay, OnConVarChange);
	
	decl String:buffer[32];
	g_hColor = CreateConVar("shop_zr_infect_nade_color", "75 255 75 255", "Infection nade spread color", _);
	GetConVarString(g_hColor, buffer, sizeof(buffer));
	StringToColor(buffer, g_iColor);
	HookConVarChange(g_hColor, OnConVarChange);
	
	g_hSound = CreateConVar("shop_zr_infect_nade_sound", "ui/freeze_cam.wav", "Infection nade explode sound");
	GetConVarString(g_hSound, g_sSound, sizeof(g_sSound));
	HookConVarChange(g_hSound, OnConVarChange);
	
	g_hPrice = CreateConVar("shop_zr_infect_nade_price", "1200", "Price for the infection nade", _, true, 1.0);
	iPrice = GetConVarInt(g_hPrice);
	HookConVarChange(g_hPrice, OnConVarChange);
	
	LoadTranslations("shop_zr_infection_nade");
	
	AutoExecConfig(true, "zombiereloaded/shop_zr_infection_nade");
	
	if (Shop_IsStarted()) Shop_Started();
}

public OnPluginEnd()
{
	Shop_UnregisterMe();
}

public OnMapStart()
{
	g_beamsprite = PrecacheModel("materials/sprites/lgtning.vmt");
	g_halosprite = PrecacheModel("materials/sprites/halo01.vmt");
}

public OnConfigsExecuted()
{
	PrecacheSound(g_sSound);
	AddFileToDownloadsTable(g_sSound);
}

/*public Action:CS_OnBuyCommand(client, const String:weapon[])
{
	if (StrContains(weapon, "smokegrenade", false) != -1)
	{
		return Plugin_Handled;
	}
	return Plugin_Continue;
}*/

public OnConVarChange(Handle:convar, const String:oldValue[], const String:newValue[])
{
	if (convar == g_hDistance)
	{
		g_fDistance = StringToFloat(newValue);
	}
	else if (convar == g_hColor)
	{
		StringToColor(newValue, g_iColor);
	}
	else if (convar == g_hExplodeDelay)
	{
		g_fExplodeDelay = StringToFloat(newValue);
	}
	else if (convar == g_hSound)
	{
		strcopy(g_sSound, sizeof(g_sSound), newValue);
		PrecacheSound(g_sSound);
		AddFileToDownloadsTable(g_sSound);
	}
	else if (convar == g_hPrice)
	{
		iPrice = StringToInt(newValue);
		if (id != INVALID_ITEM)
		{
			Shop_SetItemPrice(id, iPrice);
		}
	}
}

public Shop_Started()
{
	new CategoryId:category_id = Shop_RegisterCategory(CATEGORY, "Zombie:Reloaded", "", OnCategoryDisplay);
	if (Shop_StartItem(category_id, ITEM))
	{
		Shop_SetInfo("infect_nade", "", iPrice, _, Item_BuyOnly);
		Shop_SetCallbacks(OnItemRegistered, _, ShouldDisplay, OnDisplay, _, _, OnBuy);
		Shop_EndItem();
	}
}

public OnItemRegistered(CategoryId:category_id, const String:category[], const String:item[], ItemId:item_id)
{
	id = item_id;
}

public bool:OnCategoryDisplay(client, CategoryId:category_id, const String:category[], const String:name[], String:buffer[], maxlen)
{
	FormatEx(buffer, maxlen, "%T", "Category", client);
	return true;
}

public bool:ShouldDisplay(client, CategoryId:category_id, const String:category[], ItemId:item_id, const String:item[], ShopMenu:menu)
{
	if (!IsPlayerAlive(client))
	{
		return true;
	}
	if (ZR_IsClientZombie(client))
	{
		return true;
	}
	return false;
}

public bool:OnDisplay(client, CategoryId:category_id, const String:category[], ItemId:item_id, const String:item[], ShopMenu:menu, &bool:disabled, const String:name[], String:buffer[], maxlen)
{
	FormatEx(buffer, maxlen, "%T", "Infection nade", client);
	return true;
}

public bool:OnBuy(client, CategoryId:category_id, const String:category[], ItemId:item_id, const String:item[], ItemType:type, price, sell_price, value)
{
	if (!IsPlayerAlive(client) || !ZR_IsClientZombie(client))
	{
		CPrintToChat(client, "%t", "Must be zombie");
		return false;
	}
	
	EquipPlayerWeapon(client, GivePlayerItem(client, "weapon_smokegrenade"));
	CPrintToChat(client, "%t", "Bought");
	
	return true;
}

public OnEntityCreated(entity, const String:classname[])
{
	if (entity < 0)
	{
		return;
	}
	
	if (!strcmp(classname, "smokegrenade_projectile"))
	{
		SDKHook(entity, SDKHook_Spawn, OnSmokeSpawned);
	}
}

public OnSmokeSpawned(entity)
{
	new owner = GetEntPropEnt(entity, Prop_Send, "m_hOwnerEntity");
	if (!(0< owner <= MaxClients))
	{
		return;
	}
	if (ZR_IsClientZombie(owner))
	{
		BeamFollowCreate(entity, g_iColor);
		SetEntProp(entity, Prop_Data, "m_nNextThinkTick", -1);
		CreateTimer(g_fExplodeDelay, ExplodeInfect, EntIndexToEntRef(entity), TIMER_FLAG_NO_MAPCHANGE);
	}
}

public Action:ExplodeInfect(Handle:timer, any:ref)
{
	new entity = EntRefToEntIndex(ref);
	if (entity == INVALID_ENT_REFERENCE)
	{
		return;
	}
	
	new client = GetEntPropEnt(entity, Prop_Send, "m_hThrower");
	
	decl Float:origin[3];
	GetEntPropVector(entity, Prop_Send, "m_vecOrigin", origin);
	
	decl Float:targetOrigin[3];
	for (new i = 1; i <= MaxClients; i++)
	{
		if (i == client || !IsClientInGame(i) || !IsPlayerAlive(i) || ZR_IsClientZombie(i))
		{
			continue;
		}
		
		GetClientAbsOrigin(i, targetOrigin);
		if (GetVectorDistance(origin, targetOrigin) <= g_fDistance && IsAbleToSee(origin, i))
		{
			ZR_InfectClient(i, client, false, false, false);
			CPrintToChat(i, "%t", "infected");
		}
	}
	
	origin[2] += 30.0;
	EmitAmbientSound(g_sSound, origin, entity);
	AcceptEntityInput(entity, "Kill");
	
	TE_SetupBeamRingPoint(origin, 10.0, g_fDistance, g_beamsprite, g_halosprite, 1, 10, 0.2, 100.0, 1.0, g_iColor, 0, 0);
	TE_SendToAll();
}

public bool:FilterTarget(entity, contentsMask, any:data)
{
	return (data == entity);
}

bool:IsAbleToSee(const Float:vecOrigin[3], client)
{
	decl Float:vecEyePos[3];
	GetClientEyePosition(client, vecEyePos);
	
	decl Float:mins[3], Float:maxs[3];
	GetClientMins(client, mins);
	GetClientMaxs(client, maxs);
	// Check outer 4 corners of player.
	if (IsRectangleVisible(vecEyePos, vecOrigin, mins, maxs, 1.30))
	{
		return true;
	}

	// Check inner 4 corners of player.
	if (IsRectangleVisible(vecEyePos, vecOrigin, mins, maxs, 0.65))
	{
		return true;
	}
	
	return false;
}

bool:IsRectangleVisible(const Float:start[3], const Float:end[3], const Float:mins[3], const Float:maxs[3], Float:scale=1.0)
{
	new Float:ZpozOffset = maxs[2];
	new Float:ZnegOffset = mins[2];
	new Float:WideOffset = ((maxs[0] - mins[0]) + (maxs[1] - mins[1])) / 4.0;

	// This rectangle is just a point!
	if (ZpozOffset == 0.0 && ZnegOffset == 0.0 && WideOffset == 0.0)
	{
		return IsPointVisible(start, end);
	}

	// Adjust to scale.
	ZpozOffset *= scale;
	ZnegOffset *= scale;
	WideOffset *= scale;
	
	// Prepare rotation matrix.
	decl Float:angles[3], Float:fwd[3], Float:right[3];

	SubtractVectors(start, end, fwd);
	NormalizeVector(fwd, fwd);

	GetVectorAngles(fwd, angles);
	GetAngleVectors(angles, fwd, right, NULL_VECTOR);

	decl Float:vRectangle[4][3], Float:vTemp[3];

	// If the player is on the same level as us, we can optimize by only rotating on the z-axis.
	if (FloatAbs(fwd[2]) <= 0.7071)
	{
		ScaleVector(right, WideOffset);
		
		// Corner 1, 2
		vTemp = end;
		vTemp[2] += ZpozOffset;
		AddVectors(vTemp, right, vRectangle[0]);
		SubtractVectors(vTemp, right, vRectangle[1]);
		
		// Corner 3, 4
		vTemp = end;
		vTemp[2] += ZnegOffset;
		AddVectors(vTemp, right, vRectangle[2]);
		SubtractVectors(vTemp, right, vRectangle[3]);
		
	}
	else if (fwd[2] > 0.0) // Player is below us.
	{
		fwd[2] = 0.0;
		NormalizeVector(fwd, fwd);
		
		ScaleVector(fwd, scale);
		ScaleVector(fwd, WideOffset);
		ScaleVector(right, WideOffset);
		
		// Corner 1
		vTemp = end;
		vTemp[2] += ZpozOffset;
		AddVectors(vTemp, right, vTemp);
		SubtractVectors(vTemp, fwd, vRectangle[0]);
		
		// Corner 2
		vTemp = end;
		vTemp[2] += ZpozOffset;
		SubtractVectors(vTemp, right, vTemp);
		SubtractVectors(vTemp, fwd, vRectangle[1]);
		
		// Corner 3
		vTemp = end;
		vTemp[2] += ZnegOffset;
		AddVectors(vTemp, right, vTemp);
		AddVectors(vTemp, fwd, vRectangle[2]);
		
		// Corner 4
		vTemp = end;
		vTemp[2] += ZnegOffset;
		SubtractVectors(vTemp, right, vTemp);
		AddVectors(vTemp, fwd, vRectangle[3]);
	}
	else // Player is above us.
	{
		fwd[2] = 0.0;
		NormalizeVector(fwd, fwd);
		
		ScaleVector(fwd, scale);
		ScaleVector(fwd, WideOffset);
		ScaleVector(right, WideOffset);

		// Corner 1
		vTemp = end;
		vTemp[2] += ZpozOffset;
		AddVectors(vTemp, right, vTemp);
		AddVectors(vTemp, fwd, vRectangle[0]);
		
		// Corner 2
		vTemp = end;
		vTemp[2] += ZpozOffset;
		SubtractVectors(vTemp, right, vTemp);
		AddVectors(vTemp, fwd, vRectangle[1]);
		
		// Corner 3
		vTemp = end;
		vTemp[2] += ZnegOffset;
		AddVectors(vTemp, right, vTemp);
		SubtractVectors(vTemp, fwd, vRectangle[2]);
		
		// Corner 4
		vTemp = end;
		vTemp[2] += ZnegOffset;
		SubtractVectors(vTemp, right, vTemp);
		SubtractVectors(vTemp, fwd, vRectangle[3]);
	}

	// Run traces on all corners.
	for (new i = 0; i < 4; i++)
	{
		if (IsPointVisible(start, vRectangle[i]))
		{
			return true;
		}
	}

	return false;
}

public bool:Filter_NoPlayers(entity, mask)
{
	return (entity > MaxClients && !(0 < GetEntPropEnt(entity, Prop_Data, "m_hOwnerEntity") <= MaxClients));
}

bool:IsPointVisible(const Float:start[3], const Float:end[3])
{
	TR_TraceRayFilter(start, end, MASK_SOLID, RayType_EndPoint, Filter_NoPlayers);

	return TR_GetFraction() == 1.0;
}

BeamFollowCreate(entity, color[4])
{
	TE_SetupBeamFollow(entity, g_beamsprite, 0, 1.0, 10.0, 10.0, 5, color);
	TE_SendToAll();
}

stock bool:StringToColor(const String:str[], color[4], defvalue = -1)
{
	new bool:result = false;
	decl String:Splitter[4][4];
	if (ExplodeString(str, " ", Splitter, sizeof(Splitter), sizeof(Splitter[])) == 4 && String_IsNumeric(Splitter[0]) && String_IsNumeric(Splitter[1]) && String_IsNumeric(Splitter[2]) && String_IsNumeric(Splitter[3]))
	{
		color[0] = StringToInt(Splitter[0]);
		color[1] = StringToInt(Splitter[1]);
		color[2] = StringToInt(Splitter[2]);
		color[3] = StringToInt(Splitter[3]);
		result = true;
	}
	else
	{
		color[0] = defvalue;
		color[1] = defvalue;
		color[2] = defvalue;
		color[3] = defvalue;
	}
	return result;
}

stock ColorToString(const color[4], String:buffer[], size)
{
	FormatEx(buffer, size, "%d %d %d %d", color[0], color[1], color[2], color[3]);
}

stock bool:String_IsNumeric2(const String:str[])
{
	return (String_IsNumeric(str) || String_IsFloat(str));
}

stock bool:String_IsNumeric(const String:str[])
{	
	new x=0;
	new numbersFound=0;

	if (str[x] == '+' || str[x] == '-')
		x++;

	while (str[x] != '\0')
	{
		if (IsCharNumeric(str[x]))
			numbersFound++;
		else
			return false;
		x++;
	}
	
	if (!numbersFound)
		return false;
	
	return true;
}

stock bool:String_IsFloat(const String:str[])
{	
	new x=0;
	new dotsFound=0;
	new numbersFound=0;

	if (str[x] == '+' || str[x] == '-')
		x++;

	while (str[x] != '\0')
	{

		if (IsCharNumeric(str[x]))
			numbersFound++;
		else if (str[x] == '.')
		{
			dotsFound++;
			if (dotsFound > 1)
				return false;
		}
		else
			return false;
		
		x++;
	}
	
	if (!numbersFound || !dotsFound) {
		return false;
	}
	
	return true;
}